@@ -46,6 +46,7 @@ static bool disconnect_sent = false;
 static int publish_count = 0;
 static bool ready_for_repeat = false;
 static volatile int status = STATUS_CONNECTING;
+static int connack_result = 0;
 
 #ifdef WIN32
 static uint64_t next_publish_tv;
@@ -129,6 +130,8 @@ void my_connect_callback(struct mosquitto *mosq, void *obj, int result, int flag
 	UNUSED(flags);
 	UNUSED(properties);
 
+	connack_result = result;
+
 	if(!result){
 		switch(cfg.pub_mode){
 			case MSGMODE_CMD:
@@ -232,6 +235,17 @@ int pub_stdin_line_loop(struct mosquitto *mosq)
 	mosquitto_loop_start(mosq);
 	stdin_finished = false;
 	do{
+		if(status == STATUS_CONNECTING){
+#ifdef WIN32
+			Sleep(100);
+#else
+			struct timespec ts;
+			ts.tv_sec = 0;
+			ts.tv_nsec = 100000000;
+			nanosleep(&ts, NULL);
+#endif
+		}
+
 		if(status == STATUS_CONNACK_RECVD){
 			pos = 0;
 			read_len = line_buf_len;
@@ -555,7 +569,11 @@ int main(int argc, char *argv[])
 	if(rc){
 		err_printf(&cfg, "Error: %s\n", mosquitto_strerror(rc));
 	}
-	return rc;
+	if(connack_result){
+		return connack_result;
+	}else{
+		return rc;
+	}
 
 cleanup:
 	mosquitto_lib_cleanup();
